@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2011-2024 Baptiste Daroussin <bapt@FreeBSD.org>
+ * Copyright (c) 2011-2025 Baptiste Daroussin <bapt@FreeBSD.org>
  * Copyright (c) 2011-2012 Julien Laffaye <jlaffaye@FreeBSD.org>
  * Copyright (c) 2012 Bryan Drewery <bryan@shatow.net>
  * Copyright (c) 2013 Matthew Seaman <matthew@FreeBSD.org>
@@ -32,6 +32,7 @@ pkg_new(struct pkg **pkg, pkg_t type)
 	*pkg = xcalloc(1, sizeof(struct pkg));
 	(*pkg)->type = type;
 	(*pkg)->rootfd = -1;
+	(*pkg)->list_sorted = false;
 
 	return (EPKG_OK);
 }
@@ -76,7 +77,7 @@ pkg_free(struct pkg *pkg)
 	for (int i = 0; i < PKG_NUM_SCRIPTS; i++)
 		xstring_free(pkg->scripts[i]);
 	for (int i = 0; i < PKG_NUM_LUA_SCRIPTS; i++)
-		tll_free_and_free(pkg->lua_scripts[i], free);
+		vec_free_and_free(&pkg->lua_scripts[i], free);
 
 	pkg_list_free(pkg, PKG_DEPS);
 	pkg_list_free(pkg, PKG_RDEPS);
@@ -85,27 +86,27 @@ pkg_free(struct pkg *pkg)
 	pkg_list_free(pkg, PKG_OPTIONS);
 	pkg_list_free(pkg, PKG_CONFIG_FILES);
 
-	tll_free_and_free(pkg->users, free);
+	vec_free_and_free(&pkg->users, free);
 	pkg->flags &= ~PKG_LOAD_USERS;
-	tll_free_and_free(pkg->groups, free);
+	vec_free_and_free(&pkg->groups, free);
 	pkg->flags &= ~PKG_LOAD_GROUPS;
-	tll_free_and_free(pkg->shlibs_required, free);
+	vec_free_and_free(&pkg->shlibs_required, free);
 	pkg->flags &= ~PKG_LOAD_SHLIBS_REQUIRED;
-	tll_free_and_free(pkg->shlibs_provided, free);
+	vec_free_and_free(&pkg->shlibs_provided, free);
 	pkg->flags &= ~PKG_LOAD_SHLIBS_REQUIRED;
-	tll_free_and_free(pkg->provides, free);
+	vec_free_and_free(&pkg->provides, free);
 	pkg->flags &= ~PKG_LOAD_PROVIDES;
-	tll_free_and_free(pkg->requires, free);
+	vec_free_and_free(&pkg->requires, free);
 	pkg->flags &= ~PKG_LOAD_REQUIRES;
-	tll_free_and_free(pkg->categories, free);
+	vec_free_and_free(&pkg->categories, free);
 	pkg->flags &= ~PKG_LOAD_CATEGORIES;
-	tll_free_and_free(pkg->licenses, free);
+	vec_free_and_free(&pkg->licenses, free);
 	pkg->flags &= ~PKG_LOAD_LICENSES;
 
-	tll_free_and_free(pkg->message, pkg_message_free);
-	tll_free_and_free(pkg->annotations, pkg_kv_free);
+	vec_free_and_free(&pkg->message, pkg_message_free);
+	vec_free_and_free(&pkg->annotations, pkg_kv_free);
 
-	tll_free_and_free(pkg->dir_to_del, free);
+	vec_free_and_free(&pkg->dir_to_del, free);
 
 	if (pkg->rootfd != -1)
 		close(pkg->rootfd);
@@ -246,7 +247,7 @@ pkg_set_s(struct pkg *pkg, pkg_attr attr, const char *str)
 		pkg->comment = xstrdup(str);
 		break;
 	case PKG_ATTR_MESSAGE:
-		tll_free_and_free(pkg->message, pkg_message_free);
+		vec_free_and_free(&pkg->message, pkg_message_free);
 		if (*str == '[') {
 			pkg_message_from_str(pkg, str, strlen(str));
 		} else {
@@ -405,45 +406,13 @@ pkg_each(config_files, struct pkg_config_file, config_files);
 int
 pkg_adduser(struct pkg *pkg, const char *name)
 {
-	assert(pkg != NULL);
-	assert(name != NULL && name[0] != '\0');
-
-	tll_foreach(pkg->users, u) {
-		if (!STREQ(u->item, name))
-			continue;
-		if (ctx.developer_mode) {
-			pkg_emit_error("duplicate user listing: %s, fatal (developer mode)", name);
-			return (EPKG_FATAL);
-		}
-		pkg_emit_error("duplicate user listing: %s, ignoring", name);
-		return (EPKG_OK);
-	}
-
-	tll_push_back(pkg->users, xstrdup(name));
-
-	return (EPKG_OK);
+	return (pkg_addstring(&pkg->users, name, "user"));
 }
 
 int
 pkg_addgroup(struct pkg *pkg, const char *name)
 {
-	assert(pkg != NULL);
-	assert(name != NULL && name[0] != '\0');
-
-	tll_foreach(pkg->groups, g) {
-		if (!STREQ(g->item, name))
-			continue;
-		if (ctx.developer_mode) {
-			pkg_emit_error("duplicate group listing: %s, fatal (developer mode)", name);
-			return (EPKG_FATAL);
-		}
-		pkg_emit_error("duplicate group listing: %s, ignoring", name);
-		return (EPKG_OK);
-	}
-
-	tll_push_back(pkg->groups, xstrdup(name));
-
-	return (EPKG_OK);
+	return (pkg_addstring(&pkg->groups, name, "group"));
 }
 
 int
@@ -601,14 +570,12 @@ pkg_addconfig_file(struct pkg *pkg, const char *path, const char *content)
 }
 
 int
-pkg_addstring(stringlist_t *list, const char *val, const char *title)
+pkg_addstring(charv_t *list, const char *val, const char *title)
 {
 	assert(val != NULL);
 	assert(title != NULL);
 
-	tll_foreach(*list, v) {
-		if (!STREQ(v->item, val))
-			continue;
+	if (charv_contains(list, val, false)) {
 		if (ctx.developer_mode) {
 			pkg_emit_error("duplicate %s listing: %s, fatal"
 			    " (developer mode)", title, val);
@@ -619,7 +586,7 @@ pkg_addstring(stringlist_t *list, const char *val, const char *title)
 		return (EPKG_OK);
 	}
 
-	tll_push_back(*list, xstrdup(val));
+	vec_push(list, xstrdup(val));
 
 	return (EPKG_OK);
 }
@@ -696,7 +663,7 @@ pkg_add_lua_script(struct pkg *pkg, const char *data, pkg_lua_script type)
 	if (type >= PKG_LUA_UNKNOWN)
 		return (EPKG_FATAL);
 
-	tll_push_back(pkg->lua_scripts[type], xstrdup(data));
+	vec_push(&pkg->lua_scripts[type], xstrdup(data));
 
 	return (EPKG_OK);
 }
@@ -906,10 +873,8 @@ pkg_shlib_flags_from_abi(const struct pkg_abi *shlib_abi)
 	enum pkg_shlib_flags flags = PKG_SHLIB_FLAGS_NONE;
 
 	if (ctx.abi.os == PKG_OS_FREEBSD) {
-		if (shlib_abi->os == PKG_OS_LINUX &&
-		    pkg_object_bool(pkg_config_get("TRACK_LINUX_COMPAT_SHLIBS"))) {
+		if (shlib_abi->os == PKG_OS_LINUX && ctx.track_linux_compat_shlibs)
 			flags |= PKG_SHLIB_FLAGS_COMPAT_LINUX;
-		}
 
 		switch (ctx.abi.arch) {
 		case PKG_ARCH_AMD64:
@@ -968,23 +933,13 @@ pkg_addshlib_required(struct pkg *pkg, const char *name,
 
 	char *full_name = pkg_shlib_name_with_flags(name, flags);
 
-	if (match_ucl_lists(full_name,
-	    pkg_config_get("SHLIB_REQUIRE_IGNORE_GLOB"),
-	    pkg_config_get("SHLIB_REQUIRE_IGNORE_REGEX"))) {
-		dbg(3, "ignoring shlib %s required by package %s", full_name, pkg->name);
+	/* silently ignore duplicates in case of shlibs */
+	if (charv_contains(&pkg->shlibs_required, full_name, false)) {
 		free(full_name);
 		return (EPKG_OK);
 	}
 
-	/* silently ignore duplicates in case of shlibs */
-	tll_foreach(pkg->shlibs_required, s) {
-		if (STREQ(s->item, full_name)) {
-			free(full_name);
-			return (EPKG_OK);
-		}
-	}
-
-	tll_push_back(pkg->shlibs_required, full_name);
+	vec_push(&pkg->shlibs_required, full_name);
 
 	dbg(3, "added shlib deps for %s on %s", pkg->name, full_name);
 
@@ -998,21 +953,15 @@ pkg_addshlib_provided(struct pkg *pkg, const char *name,
 	assert(pkg != NULL);
 	assert(name != NULL && name[0] != '\0');
 
-	/* ignore files which are not starting with lib */
-	if (strncmp(name, "lib", 3) != 0)
-		return (EPKG_OK);
-
 	char *full_name = pkg_shlib_name_with_flags(name, flags);
 
 	/* silently ignore duplicates in case of shlibs */
-	tll_foreach(pkg->shlibs_provided, s) {
-		if (STREQ(s->item, full_name)) {
-			free(full_name);
-			return (EPKG_OK);
-		}
+	if (charv_contains(&pkg->shlibs_provided, full_name, false)) {
+		free(full_name);
+		return (EPKG_OK);
 	}
 
-	tll_push_back(pkg->shlibs_provided, full_name);
+	vec_push(&pkg->shlibs_provided, full_name);
 
 	dbg(3, "added shlib provide %s for %s", full_name, pkg->name);
 
@@ -1049,12 +998,10 @@ pkg_addrequire(struct pkg *pkg, const char *name)
 	assert(name != NULL && name[0] != '\0');
 
 	/* silently ignore duplicates in case of conflicts */
-	tll_foreach(pkg->requires, p) {
-		if (STREQ(p->item, name))
-			return (EPKG_OK);
-	}
+	if (charv_contains(&pkg->requires, name, false))
+		return (EPKG_OK);
 
-	tll_push_back(pkg->requires, xstrdup(name));
+	vec_push(&pkg->requires, xstrdup(name));
 
 	return (EPKG_OK);
 }
@@ -1066,12 +1013,10 @@ pkg_addprovide(struct pkg *pkg, const char *name)
 	assert(name != NULL && name[0] != '\0');
 
 	/* silently ignore duplicates in case of conflicts */
-	tll_foreach(pkg->provides, p) {
-		if (STREQ(p->item, name))
-			return (EPKG_OK);
-	}
+	if (charv_contains(&pkg->provides, name, false))
+		return (EPKG_OK);
 
-	tll_push_back(pkg->provides, xstrdup(name));
+	vec_push(&pkg->provides, xstrdup(name));
 
 	return (EPKG_OK);
 }
@@ -1081,9 +1026,9 @@ pkg_kv_get(const kvlist_t *kv, const char *tag)
 {
 	assert(tag != NULL);
 
-	tll_foreach(*kv, k) {
-		if (STREQ(k->item->key, tag))
-			return (k->item->value);
+	vec_foreach(*kv, i) {
+		if (STREQ(kv->d[i]->key, tag))
+			return (kv->d[i]->value);
 	}
 
 	return (NULL);
@@ -1097,8 +1042,8 @@ pkg_kv_add(kvlist_t *list, const char *key, const char *val, const char *title)
 	assert(val != NULL);
 	assert(title != NULL);
 
-	tll_foreach(*list, k) {
-		if (!STREQ(k->item->key, key))
+	vec_foreach(*list, i) {
+		if (!STREQ(list->d[i]->key, key))
 			continue;
 		if (ctx.developer_mode) {
 			pkg_emit_error("duplicate %s: %s, fatal"
@@ -1111,7 +1056,7 @@ pkg_kv_add(kvlist_t *list, const char *key, const char *val, const char *title)
 	}
 
 	kv = pkg_kv_new(key, val);
-	tll_push_back(*list, kv);
+	vec_push(list, kv);
 
 	return (EPKG_OK);
 }
@@ -1135,17 +1080,17 @@ pkg_list_count(const struct pkg *pkg, pkg_list list)
 	case PKG_CONFIG_FILES:
 		return (pkghash_count(pkg->config_files_hash));
 	case PKG_USERS:
-		return (tll_length(pkg->users));
+		return (vec_len(&pkg->users));
 	case PKG_GROUPS:
-		return (tll_length(pkg->groups));
+		return (vec_len(&pkg->groups));
 	case PKG_SHLIBS_REQUIRED:
-		return (tll_length(pkg->shlibs_required));
+		return (vec_len(&pkg->shlibs_required));
 	case PKG_SHLIBS_PROVIDED:
-		return (tll_length(pkg->shlibs_provided));
+		return (vec_len(&pkg->shlibs_provided));
 	case PKG_REQUIRES:
-		return (tll_length(pkg->requires));
+		return (vec_len(&pkg->requires));
 	case PKG_PROVIDES:
-		return (tll_length(pkg->provides));
+		return (vec_len(&pkg->provides));
 	}
 
 	return (0);
@@ -1464,7 +1409,7 @@ pkg_is_installed(struct pkgdb *db, const char *name)
 bool
 pkg_has_message(struct pkg *p)
 {
-	return (tll_length(p->message) > 0);
+	return (vec_len(&p->message) > 0);
 }
 
 bool
@@ -1561,7 +1506,7 @@ pkg_message_from_ucl(struct pkg *pkg, const ucl_object_t *obj)
 		msg = xcalloc(1, sizeof(*msg));
 		msg->str = xstrdup(ucl_object_tostring(obj));
 		msg->type = PKG_MESSAGE_ALWAYS;
-		tll_push_back(pkg->message, msg);
+		vec_push(&pkg->message, msg);
 		return (EPKG_OK);
 	}
 
@@ -1597,7 +1542,7 @@ pkg_message_from_ucl(struct pkg *pkg, const ucl_object_t *obj)
 				    " message will always be printed");
 		}
 		if (msg->type != PKG_MESSAGE_UPGRADE) {
-			tll_push_back(pkg->message, msg);
+			vec_push(&pkg->message, msg);
 			continue;
 		}
 
@@ -1611,7 +1556,7 @@ pkg_message_from_ucl(struct pkg *pkg, const ucl_object_t *obj)
 			msg->maximum_version = xstrdup(ucl_object_tostring(elt));
 		}
 
-		tll_push_back(pkg->message, msg);
+		vec_push(&pkg->message, msg);
 	}
 
 	return (EPKG_OK);
@@ -1667,8 +1612,8 @@ pkg_message_to_ucl(const struct pkg *pkg)
 	ucl_object_t *obj;
 
 	array = ucl_object_typed_new(UCL_ARRAY);
-	tll_foreach(pkg->message, t) {
-		msg = t->item;
+	vec_foreach(pkg->message, i) {
+		msg = pkg->message.d[i];
 		obj = ucl_object_typed_new (UCL_OBJECT);
 
 		ucl_object_insert_key(obj,
@@ -1717,7 +1662,7 @@ pkg_message_to_str(struct pkg *pkg)
 	ucl_object_t *obj;
 	char *ret = NULL;
 
-	if (tll_length(pkg->message) <= 0)
+	if (vec_len(&pkg->message) <= 0)
 		return (NULL);
 
 	obj = pkg_message_to_ucl(pkg);
@@ -1756,9 +1701,35 @@ pkg_cf_cmp(struct pkg_config_file *a, struct pkg_config_file *b)
 {
 	return (STREQ(a->path, b->path));
 }
+
+static int
+char_cmp(const void *a, const void *b) {
+    return strcmp(*(char **)a, *(char **)b);
+}
+
 void
 pkg_lists_sort(struct pkg *p)
 {
+	if (p->list_sorted)
+		return;
+	p->list_sorted = true;
+
+	if (p->categories.d)
+		qsort(p->categories.d, p->categories.len, sizeof(char *), char_cmp);
+	if (p->licenses.d)
+		qsort(p->licenses.d, p->licenses.len, sizeof(char *), char_cmp);
+	if (p->users.d)
+		qsort(p->users.d, p->users.len, sizeof(char *), char_cmp);
+	if (p->groups.d)
+		qsort(p->groups.d, p->groups.len, sizeof(char *), char_cmp);
+	if (p->shlibs_required.d)
+		qsort(p->shlibs_required.d, p->shlibs_required.len, sizeof(char *), char_cmp);
+	if (p->shlibs_provided.d)
+		qsort(p->shlibs_provided.d, p->shlibs_provided.len, sizeof(char *), char_cmp);
+	if (p->provides.d)
+		qsort(p->provides.d, p->provides.len, sizeof(p->provides.d[0]), char_cmp);
+	if (p->requires.d)
+		qsort(p->requires.d, p->requires.len, sizeof(p->requires.d[0]), char_cmp);
 	DL_SORT(p->depends, pkg_dep_cmp);
 	DL_SORT(p->files, pkg_file_cmp);
 	DL_SORT(p->dirs, pkg_dir_cmp);
