@@ -26,7 +26,6 @@
 #include <sys/wait.h>
 #include <time.h>
 #include <xstring.h>
-#include <tllist.h>
 
 #include "pkg.h"
 #include "private/event.h"
@@ -41,6 +40,17 @@
 #define NOCHANGESFLAGS	(UF_IMMUTABLE | UF_APPEND | SF_IMMUTABLE | SF_APPEND)
 #endif
 
+struct pkg_add_db {
+	struct pkgdb *db;
+	pkgs_t localpkgs;
+	struct pkghash *system_shlibs;
+	bool local_scaned;
+	bool ignore_compat32;
+	bool pkgbase;
+};
+
+static int pkg_add_common(struct pkg_add_db *db, const char *path, unsigned flags, const char *reloc, struct pkg *remote, struct pkg *local, struct triggers *t);
+
 struct external_merge_tmp_file {
 	int fd;
 	const char *template;
@@ -407,7 +417,7 @@ get_tempdir(struct pkg_add_context *context, const char *path, tempdirs_t *tempd
 {
 	struct tempdir *tmpdir = NULL;
 
-	for (size_t i = 0; i < tempdirs->len; i++) {
+	vec_foreach(*tempdirs, i) {
 		tmpdir = tempdirs->d[i];
 		if (strncmp(tmpdir->name, path, tmpdir->len) == 0 && path[tmpdir->len] == '/') {
 			reopen_tempdir(context->rootfd, tmpdir);
@@ -630,7 +640,7 @@ create_hardlink(struct pkg_add_context *context, struct pkg_file *f, const char
 		return (EPKG_FATAL);
 	}
 	if (fh->temppath[0] == '\0') {
-		for (size_t i = 0; i < tempdirs->len; i++) {
+		vec_foreach(*tempdirs, i) {
 			if (strncmp(tempdirs->d[i]->name, fh->path, tempdirs->d[i]->len) == 0 &&
 			    fh->path[tempdirs->d[i]->len] == '/' ) {
 				tmphdir = tempdirs->d[i];
@@ -995,7 +1005,7 @@ pkg_extract_finalize(struct pkg *pkg, tempdirs_t *tempdirs)
 
 
 	if (tempdirs != NULL) {
-		for (size_t i = 0; i < tempdirs->len; i++) {
+		vec_foreach(*tempdirs, i) {
 			struct tempdir *t = tempdirs->d[i];
 			if (renameat(pkg->rootfd, RELATIVE_PATH(t->temp),
 			    pkg->rootfd, RELATIVE_PATH(t->name)) != 0) {
@@ -1071,59 +1081,91 @@ pkg_extract_finalize(struct pkg *pkg, tempdirs_t *tempdirs)
 	return (EPKG_OK);
 }
 
-static char *
-pkg_globmatch(char *pattern, const char *name)
+static bool
+should_append_pkg(pkgs_t *localpkgs, struct pkg *p)
 {
-	glob_t g;
-	int i;
-	char *buf, *buf2;
-	char *path = NULL;
-
-	if (glob(pattern, 0, NULL, &g) == GLOB_NOMATCH) {
-		globfree(&g);
-
-		return (NULL);
-	}
-
-	for (i = 0; i < g.gl_pathc; i++) {
-		/* the version starts here */
-		buf = strrchr(g.gl_pathv[i], '-');
-		if (buf == NULL)
-			continue;
-		buf2 = strrchr(g.gl_pathv[i], '/');
-		if (buf2 == NULL)
-			buf2 = g.gl_pathv[i];
-		else
-			buf2++;
-		/* ensure we have match the proper name */
-		if (strncmp(buf2, name, buf - buf2) != 0)
-			continue;
-		if (path == NULL) {
-			path = g.gl_pathv[i];
-			continue;
+	/* only keep the highest version is we fine one */
+	vec_foreach(*localpkgs, i) {
+		struct pkg *lp = localpkgs->d[i];
+		if (strcmp(lp->name, p->name) == 0) {
+			if (pkg_version_cmp(lp->version, p->version) == -1) {
+				pkg_free(localpkgs->d[i]);
+				localpkgs->d[i] = p;
+				return (true);
+			}
+			return (false);
 		}
-		if (pkg_version_cmp(path, g.gl_pathv[i]) == 1)
-			path = g.gl_pathv[i];
 	}
-	if (path)
-		path = xstrdup(path);
-	globfree(&g);
+	/* none found we should append */
+	vec_push(localpkgs, p);
+	return (true);
+}
 
-	return (path);
+struct localhashes {
+	struct pkghash *shlibs_provides;
+	struct pkghash *provides;
+};
+
+static void
+scan_local_pkgs(struct pkg_add_db *db, bool fromstdin, struct localhashes *l, const char *bd, const char *ext)
+{
+	if (!fromstdin) {
+		if (!db->local_scaned) {
+			glob_t g;
+			char *pattern;
+			xasprintf(&pattern, "%s/*%s" , bd, ext);
+			db->local_scaned = true;
+			if (glob(pattern, 0, NULL, &g) == 0) {
+				for (int i = 0; i <g.gl_pathc; i++) {
+					struct pkg *p = NULL;
+					if (pkg_open(&p, g.gl_pathv[i],
+					    PKG_OPEN_MANIFEST_COMPACT) == EPKG_OK) {
+						if (should_append_pkg(&db->localpkgs, p)) {
+							p->repopath = xstrdup(g.gl_pathv[i]);
+						}
+					}
+				}
+			}
+			globfree(&g);
+			free(pattern);
+		}
+		vec_foreach(db->localpkgs, i) {
+			struct pkg *p = db->localpkgs.d[i];
+			vec_foreach(p->shlibs_provided, j) {
+				pkghash_safe_add(l->shlibs_provides, p->shlibs_provided.d[j], xstrdup(p->repopath), free);
+			}
+			vec_foreach(p->provides, j) {
+				pkghash_safe_add(l->provides, p->provides.d[j], xstrdup(p->repopath), free);
+			}
+		}
+	}
+}
+
+static const char *
+_localpkgs_get(pkgs_t *pkgs, const char *name)
+{
+	vec_foreach(*pkgs, i) {
+		if (STREQ(pkgs->d[i]->name, name))
+			return (pkgs->d[i]->repopath);
+	}
+	return (NULL);
 }
 
 static int
-pkg_add_check_pkg_archive(struct pkgdb *db, struct pkg *pkg,
+pkg_add_check_pkg_archive(struct pkg_add_db *db, struct pkg *pkg,
 	const char *path, int flags, const char *location)
 {
 	const char	*arch;
 	int	ret, retcode;
 	struct pkg_dep	*dep = NULL;
 	char	bd[MAXPATHLEN];
-	char	dpath[MAXPATHLEN], *ppath;
 	const char	*ext = NULL;
 	struct pkg	*pkg_inst = NULL;
 	bool	fromstdin;
+	struct localhashes l;
+	bool scanned = false;
+
+	memset(&l, 0, sizeof(l));
 
 	arch = pkg->abi != NULL ? pkg->abi : pkg->altabi;
 
@@ -1136,7 +1178,7 @@ pkg_add_check_pkg_archive(struct pkgdb *db, struct pkg *pkg,
 	}
 
 	/* XX check */
-	ret = pkg_try_installed(db, pkg->name, &pkg_inst, PKG_LOAD_BASIC);
+	ret = pkg_try_installed(db->db, pkg->name, &pkg_inst, PKG_LOAD_BASIC);
 	if (ret == EPKG_OK) {
 		if ((flags & PKG_ADD_FORCE) == 0) {
 			pkg_emit_already_installed(pkg_inst);
@@ -1185,9 +1227,9 @@ pkg_add_check_pkg_archive(struct pkgdb *db, struct pkg *pkg,
 	pkg_emit_add_deps_begin(pkg);
 
 	while (pkg_deps(pkg, &dep) == EPKG_OK) {
-		dpath[0] = '\0';
+		const char *founddep = NULL;
 
-		if (pkg_is_installed(db, dep->name) == EPKG_OK)
+		if (pkg_is_installed(db->db, dep->name) == EPKG_OK)
 			continue;
 
 		if (fromstdin) {
@@ -1197,28 +1239,102 @@ pkg_add_check_pkg_archive(struct pkgdb *db, struct pkg *pkg,
 			continue;
 		}
 
-		if (dep->version != NULL && dep->version[0] != '\0') {
-			snprintf(dpath, sizeof(dpath), "%s/%s-%s%s", bd,
-				dep->name, dep->version, ext);
+		if (!scanned) {
+			scan_local_pkgs(db, fromstdin, &l, bd, ext);
+			scanned = true;
 		}
-
-		if (strlen(dpath) == 0 || access(dpath, F_OK) != 0) {
-			snprintf(dpath, sizeof(dpath), "%s/%s-*%s", bd,
-			    dep->name, ext);
-			ppath = pkg_globmatch(dpath, dep->name);
-			if (ppath == NULL) {
-				pkg_emit_missing_dep(pkg, dep);
-				if ((flags & PKG_ADD_FORCE_MISSING) == 0)
-					goto cleanup;
-				continue;
-			}
-			strlcpy(dpath, ppath, sizeof(dpath));
-			free(ppath);
+		if ((founddep = _localpkgs_get(&db->localpkgs, dep->name)) == NULL) {
+			pkg_emit_missing_dep(pkg, dep);
+			if ((flags & PKG_ADD_FORCE_MISSING) == 0)
+				goto cleanup;
+			continue;
 		}
 
 		if ((flags & PKG_ADD_UPGRADE) == 0 &&
-				access(dpath, F_OK) == 0) {
-			ret = pkg_add(db, dpath, PKG_ADD_AUTOMATIC, location);
+				access(founddep, F_OK) == 0) {
+			pkg_debug(2, "Installing %s because of direct dependency: %s", founddep, dep->name);
+			ret = pkg_add_common(db, founddep, PKG_ADD_AUTOMATIC, location, NULL, NULL, NULL);
+
+			if (ret != EPKG_OK)
+				goto cleanup;
+		} else {
+			pkg_emit_missing_dep(pkg, dep);
+			if ((flags & PKG_ADD_FORCE_MISSING) == 0)
+				goto cleanup;
+		}
+	}
+
+	vec_foreach(pkg->shlibs_required, i) {
+		const char *s = pkg->shlibs_required.d[i];
+		pkg_debug(2, "%s requires %s", pkg->name, s);
+		if (!db->pkgbase && db->system_shlibs == NULL) {
+			int ret;
+			ret = scan_system_shlibs(&db->system_shlibs, ctx.pkg_rootdir);
+			if (ret == EPKG_NOCOMPAT32)
+				db->ignore_compat32 = true;
+		}
+		if (pkghash_get(db->system_shlibs, s) != NULL)
+			continue;
+		pkghash_entry *founddep = NULL;
+		if (pkgdb_is_shlib_provided(db->db, s))
+			continue;
+
+		if (fromstdin) {
+			pkg_emit_error("Missing shlib dependency: %s", s);
+			if ((flags & PKG_ADD_FORCE_MISSING) == 0)
+				goto cleanup;
+			continue;
+		}
+		if (!scanned) {
+			scan_local_pkgs(db, fromstdin, &l, bd, ext);
+			scanned = true;
+		}
+		if ((founddep = pkghash_get(l.shlibs_provides, s)) == NULL) {
+			pkg_emit_error("Missing shlib dependency: %s", s);
+			if ((flags & PKG_ADD_FORCE_MISSING) == 0)
+				goto cleanup;
+			continue;
+		}
+		if ((flags & PKG_ADD_UPGRADE) == 0 &&
+				access(founddep->value, F_OK) == 0) {
+			pkg_debug(2, "Installing %s because of shlibs_required: %s", founddep->value, s);
+			ret = pkg_add_common(db, founddep->value, PKG_ADD_AUTOMATIC, location, NULL, NULL, NULL);
+
+			if (ret != EPKG_OK)
+				goto cleanup;
+		} else {
+			pkg_emit_missing_dep(pkg, dep);
+			if ((flags & PKG_ADD_FORCE_MISSING) == 0)
+				goto cleanup;
+		}
+	}
+
+	vec_foreach(pkg->requires, i) {
+		const char *s = pkg->requires.d[i];
+		pkghash_entry *founddep = NULL;
+		if (pkgdb_is_provided(db->db, s))
+			continue;
+
+		if (fromstdin) {
+			pkg_emit_error("Missing require dependency: %s", s);
+			if ((flags & PKG_ADD_FORCE_MISSING) == 0)
+				goto cleanup;
+			continue;
+		}
+		if (!scanned) {
+			scan_local_pkgs(db, fromstdin, &l, bd, ext);
+			scanned = true;
+		}
+		if ((founddep = pkghash_get(l.provides, s)) == NULL) {
+			pkg_emit_error("Missing require dependency: %s", s);
+			if ((flags & PKG_ADD_FORCE_MISSING) == 0)
+				goto cleanup;
+			continue;
+		}
+		if ((flags & PKG_ADD_UPGRADE) == 0 &&
+				access(founddep->value, F_OK) == 0) {
+			pkg_debug(2, "Installing %s because of requires: %s", founddep, s);
+			ret = pkg_add_common(db, founddep->value, PKG_ADD_AUTOMATIC, location, NULL, NULL, NULL);
 
 			if (ret != EPKG_OK)
 				goto cleanup;
@@ -1231,6 +1347,8 @@ pkg_add_check_pkg_archive(struct pkgdb *db, struct pkg *pkg,
 
 	retcode = EPKG_OK;
 cleanup:
+	pkghash_destroy(l.provides);
+	pkghash_destroy(l.shlibs_provides);
 	pkg_emit_add_deps_finished(pkg);
 
 	return (retcode);
@@ -1248,12 +1366,14 @@ pkg_add_cleanup_old(struct pkgdb *db, struct pkg *old, struct pkg *new, struct t
 	 * Execute pre deinstall scripts
 	 */
 	if ((flags & PKG_ADD_NOSCRIPT) == 0) {
+		bool noexec = ((flags & PKG_ADD_NOEXEC) == PKG_ADD_NOEXEC);
 		ret = pkg_lua_script_run(old, PKG_LUA_PRE_DEINSTALL, (old != NULL));
 		if (ret != EPKG_OK && ctx.developer_mode) {
 			return (ret);
 		} else {
-			ret = pkg_script_run(old, PKG_SCRIPT_PRE_DEINSTALL, (old != NULL));
-			if (ret != EPKG_OK && ctx.developer_mode) {
+			ret = pkg_script_run(old, PKG_SCRIPT_PRE_DEINSTALL, (old != NULL),
+			    noexec);
+			if (ret != EPKG_OK && (ctx.developer_mode || noexec)) {
 				return (ret);
 			} else {
 				ret = EPKG_OK;
@@ -1273,7 +1393,7 @@ pkg_add_cleanup_old(struct pkgdb *db, struct pkg *old, struct pkg *new, struct t
 					const char *libname;
 					libname = strrchr(f->path, '/');
 					if (libname != NULL &&
-					    stringlist_contains(&old->shlibs_provided, libname+1)) {
+					    charv_contains(&old->shlibs_provided, libname+1, false)) {
 						backup_library(db, old, f->path);
 					}
 				}
@@ -1318,7 +1438,7 @@ pkg_add_triggers(void)
 }
 
 static int
-pkg_add_common(struct pkgdb *db, const char *path, unsigned flags,
+pkg_add_common(struct pkg_add_db *db, const char *path, unsigned flags,
     const char *reloc, struct pkg *remote,
     struct pkg *local, struct triggers *t)
 {
@@ -1356,8 +1476,6 @@ pkg_add_common(struct pkgdb *db, const char *path, unsigned flags,
 		retcode = ret;
 		goto cleanup;
 	}
-	if ((flags & PKG_ADD_SPLITTED_UPGRADE) == 0)
-		pkg_emit_new_action();
 	if ((flags & (PKG_ADD_UPGRADE | PKG_ADD_SPLITTED_UPGRADE)) !=
 	    PKG_ADD_UPGRADE)
 		pkg_emit_install_begin(pkg);
@@ -1412,7 +1530,7 @@ pkg_add_common(struct pkgdb *db, const char *path, unsigned flags,
 		}
 		if (faccessat(pkg->rootfd, RELATIVE_PATH(f->path), F_OK, 0) == 0) {
 			f->previous = PKG_FILE_EXIST;
-			if (!pkgdb_file_exists(db, f->path)) {
+			if (!pkgdb_file_exists(db->db, f->path)) {
 				f->previous = PKG_FILE_SAVE;
 			}
 		}
@@ -1422,7 +1540,7 @@ pkg_add_common(struct pkgdb *db, const char *path, unsigned flags,
 	 * Register the package before installing it in case there are problems
 	 * that could be caught here.
 	 */
-	retcode = pkgdb_register_pkg(db, pkg, flags & PKG_ADD_FORCE, NULL);
+	retcode = pkgdb_register_pkg(db->db, pkg, flags & PKG_ADD_FORCE, NULL);
 	if (retcode != EPKG_OK)
 		goto cleanup;
 	openxact = true;
@@ -1433,7 +1551,8 @@ pkg_add_common(struct pkgdb *db, const char *path, unsigned flags,
 	if ((flags & PKG_ADD_NOSCRIPT) == 0) {
 		if ((retcode = pkg_lua_script_run(pkg, PKG_LUA_PRE_INSTALL, (local != NULL))) != EPKG_OK)
 			goto cleanup;
-		if ((retcode = pkg_script_run(pkg, PKG_SCRIPT_PRE_INSTALL, (local != NULL))) != EPKG_OK)
+		if ((retcode = pkg_script_run(pkg, PKG_SCRIPT_PRE_INSTALL, (local != NULL),
+			    ((flags & PKG_ADD_NOEXEC) == PKG_ADD_NOEXEC))) != EPKG_OK)
 			goto cleanup;
 	}
 
@@ -1451,7 +1570,7 @@ pkg_add_common(struct pkgdb *db, const char *path, unsigned flags,
 		if (retcode != EPKG_OK) {
 			/* If the add failed, clean up (silently) */
 			pkg_rollback_pkg(pkg);
-			pkg_delete_dirs(db, pkg, NULL);
+			pkg_delete_dirs(db->db, pkg, NULL);
 			goto cleanup;
 		}
 	}
@@ -1463,7 +1582,7 @@ pkg_add_common(struct pkgdb *db, const char *path, unsigned flags,
 	if (local != NULL && (flags & PKG_ADD_SPLITTED_UPGRADE) == 0) {
 		pkg_open_root_fd(local);
 		pkg_debug(1, "Cleaning up old version");
-		if (pkg_add_cleanup_old(db, local, pkg, t, flags) != EPKG_OK) {
+		if (pkg_add_cleanup_old(db->db, local, pkg, t, flags) != EPKG_OK) {
 			retcode = EPKG_FATAL;
 			goto cleanup;
 		}
@@ -1471,11 +1590,11 @@ pkg_add_common(struct pkgdb *db, const char *path, unsigned flags,
 
 
 	/* Update configuration file content with db with newer versions */
-	pkgdb_update_config_file_content(pkg, db->sqlite);
+	pkgdb_update_config_file_content(pkg, db->db->sqlite);
 
 	retcode = pkg_extract_finalize(pkg, &tempdirs);
 
-	pkgdb_register_finale(db, retcode, NULL);
+	pkgdb_register_finale(db->db, retcode, NULL);
 	openxact = false;
 
 	/*
@@ -1485,8 +1604,13 @@ pkg_add_common(struct pkgdb *db, const char *path, unsigned flags,
 	if (retcode != EPKG_OK)
 		goto cleanup;
 	if ((flags & PKG_ADD_NOSCRIPT) == 0) {
+		bool noexec = ((flags & PKG_ADD_NOEXEC) == PKG_ADD_NOEXEC);
 		pkg_lua_script_run(pkg, PKG_LUA_POST_INSTALL, (local != NULL));
-		pkg_script_run(pkg, PKG_SCRIPT_POST_INSTALL, (local != NULL));
+		retcode = pkg_script_run(pkg, PKG_SCRIPT_POST_INSTALL, (local != NULL),
+		    noexec);
+		if (retcode != EPKG_OK && noexec)
+			goto cleanup;
+		retcode = EPKG_OK;
 	}
 
 	/*
@@ -1502,8 +1626,8 @@ pkg_add_common(struct pkgdb *db, const char *path, unsigned flags,
 	else
 		pkg_emit_upgrade_finished(pkg, local);
 
-	tll_foreach(pkg->message, m) {
-		msg = m->item;
+	vec_foreach(pkg->message, i) {
+		msg = pkg->message.d[i];
 		msgstr = NULL;
 		if (msg->type == PKG_MESSAGE_ALWAYS) {
 			msgstr = msg->str;
@@ -1545,7 +1669,7 @@ pkg_add_common(struct pkgdb *db, const char *path, unsigned flags,
 
 cleanup:
 	if (openxact)
-		pkgdb_register_finale(db, retcode, NULL);
+		pkgdb_register_finale(db->db, retcode, NULL);
 	if (a != NULL) {
 		archive_read_close(a);
 		archive_read_free(a);
@@ -1560,14 +1684,33 @@ int
 pkg_add(struct pkgdb *db, const char *path, unsigned flags,
     const char *location)
 {
-	return pkg_add_common(db, path, flags, location, NULL, NULL, NULL);
+	struct pkg_add_db padb;
+	int ret;
+
+	memset(&padb, 0, sizeof(padb));
+	padb.db = db;
+	padb.local_scaned = false;
+	padb.ignore_compat32 = false;
+	padb.pkgbase = pkgdb_file_exists(db, "/usr/bin/uname");
+
+	ret = pkg_add_common(&padb, path, flags, location, NULL, NULL, NULL);
+	vec_free_and_free(&padb.localpkgs, pkg_free);
+	pkghash_destroy(padb.system_shlibs);
+	return (ret);
 }
 
 int
 pkg_add_from_remote(struct pkgdb *db, const char *path, unsigned flags,
     const char *location, struct pkg *rp, struct triggers *t)
 {
-	return pkg_add_common(db, path, flags, location, rp, NULL, t);
+	struct pkg_add_db padb;
+
+	memset(&padb, 0, sizeof(padb));
+	padb.db = db;
+	padb.ignore_compat32 = false;
+	padb.pkgbase = pkgdb_file_exists(db, "/usr/bin/uname");
+
+	return pkg_add_common(&padb, path, flags, location, rp, NULL, t);
 }
 
 int
@@ -1575,11 +1718,18 @@ pkg_add_upgrade(struct pkgdb *db, const char *path, unsigned flags,
     const char *location,
     struct pkg *rp, struct pkg *lp, struct triggers *t)
 {
+	struct pkg_add_db padb;
+
+	memset(&padb, 0, sizeof(padb));
+	padb.db = db;
+	padb.ignore_compat32 = false;
+	padb.pkgbase = pkgdb_file_exists(db, "/usr/bin/uname");
+
 	if (pkgdb_ensure_loaded(db, lp,
 	    PKG_LOAD_FILES|PKG_LOAD_SCRIPTS|PKG_LOAD_DIRS|PKG_LOAD_LUA_SCRIPTS) != EPKG_OK)
 		return (EPKG_FATAL);
 
-	return pkg_add_common(db, path, flags, location, rp, lp, t);
+	return pkg_add_common(&padb, path, flags, location, rp, lp, t);
 }
 
 static int
@@ -1625,7 +1775,7 @@ pkg_add_group(struct pkg *pkg)
 }
 
 int
-pkg_add_fromdir(struct pkg *pkg, const char *src, struct pkgdb *db)
+pkg_add_fromdir(struct pkg *pkg, const char *src, struct pkgdb *db __unused)
 {
 	struct stat st;
 	struct pkg_dir *d = NULL;
@@ -1788,7 +1938,7 @@ pkg_add_fromdir(struct pkg *pkg, const char *src, struct pkgdb *db)
 				    " '%s'", RELATIVE_PATH(f->path));
 			}
 			path = NULL;
-			for (size_t i = 0; i < hardlinks.len; i++) {
+			vec_foreach(hardlinks, i) {
 				struct hardlink *hit = hardlinks.d[i];
 				if (hit->ino == st.st_ino &&
 				    hit->dev == st.st_dev) {
