@@ -456,7 +456,7 @@ pkg_jobs_add_req(struct pkg_jobs *j, struct pkg *pkg)
 }
 
 static bool
-append_to_del_request(struct pkg_jobs *j, pkg_chain_t *to_process, const char *uid, const char *reqname)
+append_to_del_request(struct pkg_jobs *j, pkgs_t *to_process, const char *uid, const char *reqname)
 {
 	struct pkg *p;
 
@@ -468,7 +468,7 @@ append_to_del_request(struct pkg_jobs *j, pkg_chain_t *to_process, const char *u
 		   reqname);
 		return (false);
 	}
-	tll_push_back(*to_process, p);
+	vec_push(to_process, p);
 	return (true);
 }
 
@@ -476,13 +476,16 @@ bool
 delete_process_provides(struct pkg_jobs *j, struct pkg *lp, const char *provide,
     struct pkgdb_it *(*provideq)(struct pkgdb *db, const char *req),
     struct pkgdb_it *(*requireq)(struct pkgdb *db, const char *req),
-    pkg_chain_t *to_process)
+    pkgs_t *to_process)
 {
 	struct pkgdb_it *lit, *rit;
 	struct pkg *pkg;
 	struct pkg_job_request *req;
 	bool ret = true;
 
+	/* check for pkgbase shlibs and provides */
+	if (pkghash_get(j->system_shlibs, provide) != NULL)
+		return (ret);
 	/* if something else to provide the same thing we can safely delete */
 	lit = provideq(j->db, provide);
 	if (lit == NULL)
@@ -534,9 +537,11 @@ pkg_jobs_process_delete_request(struct pkg_jobs *j)
 	struct pkg_dep *d = NULL;
 	struct pkg *lp;
 	int rc = EPKG_OK;
-	pkg_chain_t to_process = tll_init();
+	pkgs_t to_process;
 	pkghash_it it;
 
+	vec_init(&to_process);
+
 	if (force)
 		return (EPKG_OK);
 
@@ -557,15 +562,15 @@ pkg_jobs_process_delete_request(struct pkg_jobs *j)
 				rc = EPKG_FATAL;
 		}
 
-		tll_foreach(lp->provides, i) {
-			if (!delete_process_provides(j, lp, i->item,
+		vec_foreach(lp->provides, i) {
+			if (!delete_process_provides(j, lp, lp->provides.d[i],
 			    pkgdb_query_provide, pkgdb_query_require,
 			    &to_process))
 				rc = EPKG_FATAL;
 		}
 
-		tll_foreach(lp->shlibs_provided, i) {
-			if (!delete_process_provides(j, lp, i->item,
+		vec_foreach(lp->shlibs_provided, i) {
+			if (!delete_process_provides(j, lp, lp->shlibs_provided.d[i],
 			    pkgdb_query_shlib_provide,
 			    pkgdb_query_shlib_require, &to_process))
 				rc = EPKG_FATAL;
@@ -575,17 +580,17 @@ pkg_jobs_process_delete_request(struct pkg_jobs *j)
 	if (rc == EPKG_FATAL)
 		return (rc);
 
-	tll_foreach(to_process, pit) {
-		lp = pit->item;
+	vec_foreach(to_process, pit) {
+		lp = to_process.d[pit];
 		if (pkg_jobs_add_req(j, lp) == NULL) {
-			tll_free(to_process);
+			vec_free(&to_process);
 			return (EPKG_FATAL);
 		}
 	}
 
-	if (tll_length(to_process) > 0)
+	if (vec_len(&to_process) > 0)
 		rc = pkg_jobs_process_delete_request(j);
-	tll_free(to_process);
+	vec_free(&to_process);
 
 	return (rc);
 }
@@ -658,8 +663,8 @@ pkg_jobs_test_automatic(struct pkg_jobs *j, struct pkg *p)
 			return (false);
 	}
 
-	tll_foreach(p->provides, i) {
-		it = pkgdb_query_require(j->db, i->item);
+	vec_foreach(p->provides, i) {
+		it = pkgdb_query_require(j->db, p->provides.d[i]);
 		if (it == NULL)
 			continue;
 		npkg = NULL;
@@ -673,8 +678,8 @@ pkg_jobs_test_automatic(struct pkg_jobs *j, struct pkg *p)
 		pkgdb_it_free(it);
 	}
 
-	tll_foreach(p->shlibs_provided, i) {
-		it = pkgdb_query_shlib_require(j->db, i->item);
+	vec_foreach(p->shlibs_provided, i) {
+		it = pkgdb_query_shlib_require(j->db, p->shlibs_provided.d[i]);
 		if (it == NULL)
 			continue;
 		npkg = NULL;
@@ -1053,8 +1058,6 @@ pkg_jobs_need_upgrade(struct pkghash *system_shlibs, struct pkg *rp, struct pkg
 	struct pkg_option *lo = NULL, *ro = NULL;
 	struct pkg_dep *ld = NULL, *rd = NULL;
 	struct pkg_conflict *lc = NULL, *rc = NULL;
-	const char **l1;
-	size_t i;
 
 	/* If no local package, then rp is obviously need to be added */
 	if (lp == NULL)
@@ -1175,116 +1178,77 @@ pkg_jobs_need_upgrade(struct pkghash *system_shlibs, struct pkg *rp, struct pkg
 	}
 
 	/* Provides */
-	if (tll_length(rp->provides) != tll_length(lp->provides)) {
+	if (vec_len(&rp->provides) != vec_len(&lp->provides)) {
 		free(rp->reason);
 		rp->reason = xstrdup("provides changed");
 		return (true);
 	}
-	l1 = xcalloc(tll_length(lp->provides), sizeof (char*));
-	i = 0;
-	tll_foreach(lp->provides, l) {
-		l1[i++] = l->item;
-	}
-	i = 0;
-	tll_foreach(rp->provides, r) {
-		if (!STREQ(r->item, l1[i])) {
+	pkg_lists_sort(lp);
+	pkg_lists_sort(rp);
+
+	vec_foreach(lp->provides, i) {
+		if (!STREQ(lp->provides.d[i], rp->provides.d[i])) {
 			free(rp->reason);
 			rp->reason = xstrdup("provides changed");
-			free(l1);
 			return (true);
 		}
 	}
-	free(l1);
 
 	/* Requires */
-	if (tll_length(rp->requires) != tll_length(lp->requires)) {
+	if (vec_len(&rp->requires) != vec_len(&lp->requires)) {
 		free(rp->reason);
 		rp->reason = xstrdup("requires changed");
 		return (true);
 	}
-	l1 = xcalloc(tll_length(lp->requires), sizeof (char*));
-	i = 0;
-	tll_foreach(lp->requires, l) {
-		l1[i++] = l->item;
-	}
-	i = 0;
-	tll_foreach(rp->requires, r) {
-		if (!STREQ(r->item, l1[i])) {
+	vec_foreach(lp->requires, i) {
+		if (!STREQ(lp->requires.d[i], rp->requires.d[i])) {
 			free(rp->reason);
 			rp->reason = xstrdup("requires changed");
-			free(l1);
 			return (true);
 		}
 	}
-	free(l1);
 
 	/* Finish by the shlibs */
-	if (tll_length(rp->shlibs_provided) != tll_length(lp->shlibs_provided)) {
+	if (vec_len(&rp->shlibs_provided) != vec_len(&lp->shlibs_provided)) {
 		free(rp->reason);
 		rp->reason = xstrdup("provided shared library changed");
 		return (true);
 	}
-	l1 = xcalloc(tll_length(lp->shlibs_provided), sizeof (char*));
-	i = 0;
-	tll_foreach(lp->shlibs_provided, l) {
-		l1[i++] = l->item;
-	}
-	i = 0;
-	tll_foreach(rp->shlibs_provided, r) {
-		if (!STREQ(r->item, l1[i])) {
+	vec_foreach(lp->shlibs_provided, i) {
+		if (!STREQ(lp->shlibs_provided.d[i], rp->shlibs_provided.d[i])) {
 			free(rp->reason);
 			rp->reason = xstrdup("provided shared library changed");
-			free(l1);
 			return (true);
 		}
-		i++;
 	}
-	free(l1);
 
-	size_t cntr = tll_length(rp->shlibs_required);
-	size_t cntl = tll_length(lp->shlibs_required);
-	if (cntr != cntl) {
+	size_t cntr = vec_len(&rp->shlibs_required);
+	size_t cntl = vec_len(&lp->shlibs_required);
+	if (cntr != cntl & system_shlibs == NULL) {
+		free(rp->reason);
+		rp->reason = xstrdup("required shared library changed");
+		return (true);
+	}
+	size_t i, j;
+
+	for (i = 0, j = 0; i < cntl && j < cntr; i++, j++) {
+		if (STREQ(lp->shlibs_required.d[i], rp->shlibs_required.d[j]))
+				continue;
 		if (system_shlibs != NULL) {
-		/*
-		 * before considering shlibs we need to check if we are running
-		 * pkgbase
-		 */
-			tll_foreach(rp->shlibs_required, r) {
-				if (pkghash_get(system_shlibs, r->item) != NULL)
-					cntr--;
+			if (pkghash_get(system_shlibs, lp->shlibs_required.d[i]) != NULL) {
+				j--;
+				continue;
 			}
-			tll_foreach(lp->shlibs_required, l) {
-				if (pkghash_get(system_shlibs, l->item) != NULL)
-					cntl--;
+			if (pkghash_get(system_shlibs, rp->shlibs_required.d[j]) != NULL) {
+				i++;
+				continue;
 			}
 		}
-		if (cntr != cntl) {
-			free(rp->reason);
-			rp->reason = xstrdup("required shared library changed");
-			return (true);
-		}
+		free(rp->reason);
+		rp->reason = xstrdup("required shared library changed");
+		return (true);
+		break;
 	}
-	l1 = xcalloc(tll_length(lp->shlibs_required), sizeof (char*));
-	i = 0;
-	tll_foreach(lp->shlibs_required, l) {
-		if (pkghash_get(system_shlibs, l->item) != NULL)
-			continue;
-		l1[i++] = l->item;
-	}
-	i = 0;
-	tll_foreach(rp->shlibs_required, r) {
-		if (pkghash_get(system_shlibs, r->item) != NULL)
-			continue;
-		if (!STREQ(r->item, l1[i])) {
-			free(rp->reason);
-			rp->reason = xstrdup("required shared library changed");
-			free(l1);
-			return (true);
-		}
-		i++;
-	}
-	free(l1);
-
 	return (false);
 }
 
@@ -2090,6 +2054,8 @@ pkg_jobs_execute(struct pkg_jobs *j)
 	struct pkg *p;
 	int retcode = EPKG_FATAL;
 	pkg_plugin_hook_t pre, post;
+	size_t total_actions;
+	size_t current_action = 0;
 
 //j->triggers.cleanup = triggers_load(true);
 	if (j->type == PKG_JOBS_INSTALL) {
@@ -2123,9 +2089,11 @@ pkg_jobs_execute(struct pkg_jobs *j)
 	if (retcode != EPKG_OK)
 		return (retcode);
 
+	total_actions = tll_length(j->jobs);
 	tll_foreach(j->jobs, _p) {
 		struct pkg_solved *ps = _p->item;
 
+		pkg_emit_new_action(++current_action, total_actions);
 		switch (ps->type) {
 		case PKG_SOLVED_DELETE:
 			if ((j->flags & PKG_FLAG_FORCE) == 0) {
