@@ -230,6 +230,8 @@ name_key(struct plist *p, char *line, struct file_attr *a __unused)
 	tmp[0] = '\0';
 	tmp++;
 	p->pkg->name = xstrdup(line);
+	free(p->pkg->uid);
+	p->pkg->uid = xstrdup(line);
 	p->pkg->version = xstrdup(tmp);
 
 	return (EPKG_OK);
@@ -516,7 +518,7 @@ populate_keywords(struct plist *p)
 		a = xmalloc(sizeof(struct action));
 		k->keyword = xstrdup(keyacts[i].key);
 		a->perform = keyacts[i].action;
-		tll_push_back(k->actions, a);
+		vec_push(&k->actions, a);
 		pkghash_safe_add(p->keywords, k->keyword, k, NULL);
 	}
 }
@@ -525,7 +527,7 @@ static void
 keyword_free(struct keyword *k)
 {
 	free(k->keyword);
-	tll_free_and_free(k->actions, free);
+	vec_free_and_free(&k->actions, free);
 	free(k);
 }
 
@@ -705,7 +707,7 @@ apply_keyword_file(ucl_object_t *obj, struct plist *p, char *line, struct file_a
 				else if (STRIEQ(ucl_object_tostring(elt), "upgrade"))
 					msg->type = PKG_MESSAGE_UPGRADE;
 			}
-			tll_push_back(p->pkg->message, msg);
+			vec_push(&p->pkg->message, msg);
 		}
 	}
 
@@ -905,8 +907,8 @@ parse_keywords(struct plist *plist, char *keyword,
 
 	k = pkghash_get_value(plist->keywords, keyword);
 	if (k != NULL) {
-		tll_foreach(k->actions, a) {
-			ret = a->item->perform(plist, line, attr);
+		vec_foreach(k->actions, i) {
+			ret = k->actions.d[i]->perform(plist, line, attr);
 			if (ret != EPKG_OK)
 				break;
 		}
@@ -1089,7 +1091,7 @@ expand_plist_variables(const char *in, kvlist_t *vars)
 	const char *cp;
 	size_t len;
 
-	if (tll_length(*vars) == 0)
+	if (vec_len(vars) == 0)
 		return (xstrdup(in));
 
 	buf = xstring_new();
@@ -1127,10 +1129,10 @@ expand_plist_variables(const char *in, kvlist_t *vars)
 		len = in - cp -1;
 		/* we have a variable */
 		bool found = false;
-		tll_foreach(*vars, i) {
-			if (strncmp(cp, i->item->key, len) != 0)
+		vec_foreach(*vars, i) {
+			if (strncmp(cp, vars->d[i]->key, len) != 0)
 				continue;
-			fputs(i->item->value, buf->fp);
+			fputs(vars->d[i]->value, buf->fp);
 			found = true;
 			in++;
 			break;
@@ -1207,15 +1209,15 @@ add_variable(struct plist *p, char *line, struct file_attr *a __unused)
 	while (*val != '\0' && isspace(*val))
 		val++;
 
-	tll_foreach(p->variables, v) {
-		if (STREQ(v->item->key, key)) {
-			free(v->item->value);
-			v->item->value = xstrdup(val);
+	vec_foreach(p->variables, i) {
+		if (STREQ(p->variables.d[i]->key, key)) {
+			free(p->variables.d[i]->value);
+			p->variables.d[i]->value = xstrdup(val);
 			return (EPKG_OK);
 		}
 	}
 	struct pkg_kv *kv = pkg_kv_new(key, val);
-	tll_push_back(p->variables, kv);
+	vec_push(&p->variables, kv);
 	return (EPKG_OK);
 }
 
@@ -1331,7 +1333,7 @@ pkg_add_port(struct pkgdb *db, struct pkg *pkg, const char *input_path,
 	if (!testing) {
 		/* Execute pre-install scripts */
 		pkg_lua_script_run(pkg, PKG_LUA_PRE_INSTALL, false);
-		pkg_script_run(pkg, PKG_SCRIPT_PRE_INSTALL, false);
+		pkg_script_run(pkg, PKG_SCRIPT_PRE_INSTALL, false, false);
 
 		if (input_path != NULL) {
 			pkg_register_cleanup_callback(pkg_rollback_cb, pkg);
@@ -1346,17 +1348,17 @@ pkg_add_port(struct pkgdb *db, struct pkg *pkg, const char *input_path,
 
 		/* Execute post-install scripts */
 		pkg_lua_script_run(pkg, PKG_LUA_POST_INSTALL, false);
-		pkg_script_run(pkg, PKG_SCRIPT_POST_INSTALL, false);
+		pkg_script_run(pkg, PKG_SCRIPT_POST_INSTALL, false, false);
 	}
 
 	if (rc == EPKG_OK) {
 		pkg_emit_install_finished(pkg, NULL);
 		if (pkg_has_message(pkg))
 			message = xstring_new();
-		tll_foreach(pkg->message, m) {
-			if (m->item->type == PKG_MESSAGE_ALWAYS ||
-			    m->item->type == PKG_MESSAGE_INSTALL) {
-				fprintf(message->fp, "%s\n", m->item->str);
+		vec_foreach(pkg->message, i) {
+			if (pkg->message.d[i]->type == PKG_MESSAGE_ALWAYS ||
+			    pkg->message.d[i]->type == PKG_MESSAGE_INSTALL) {
+				fprintf(message->fp, "%s\n", pkg->message.d[i]->str);
 			}
 		}
 		if (pkg_has_message(pkg)) {
