@@ -58,6 +58,7 @@ enum pkg_solve_rule_type {
 	PKG_RULE_REQUEST_CONFLICT,
 	PKG_RULE_REQUEST,
 	PKG_RULE_REQUIRE,
+	PKG_RULE_VITAL,
 	PKG_RULE_MAX
 };
 
@@ -68,6 +69,7 @@ static const char *rule_reasons[] = {
 	[PKG_RULE_EXPLICIT_CONFLICT] = "conflict",
 	[PKG_RULE_REQUEST] = "request",
 	[PKG_RULE_REQUIRE] = "require",
+	[PKG_RULE_VITAL] = "vital",
 	[PKG_RULE_MAX] = NULL
 };
 
@@ -249,6 +251,13 @@ pkg_print_rule_buf(struct pkg_solve_rule *rule, xstring *sb)
 					it->next ? ", " : "");
 		}
 		break;
+	case PKG_RULE_VITAL:
+		fprintf(sb->fp, "The following packages are vital: ");
+		LL_FOREACH(rule->items, it) {
+			fprintf(sb->fp, "%s-%s%s", it->var->uid, it->var->unit->pkg->version,
+					it->next ? ", " : "");
+		}
+		break;
 	default:
 		break;
 	}
@@ -299,7 +308,7 @@ pkg_solve_handle_provide (struct pkg_solve_problem *problem,
 		pkg = curvar->unit->pkg;
 
 		if (pr->is_shlib) {
-			libfound = stringlist_contains(&pkg->shlibs_provided, pr->provide);
+			libfound = charv_contains(&pkg->shlibs_provided, pr->provide, false);
 			/* Skip incompatible ABI as well */
 			if (libfound && !STREQ(pkg->abi, orig->abi)) {
 				dbg(2, "require %s: package %s-%s(%c) provides wrong ABI %s, "
@@ -309,7 +318,7 @@ pkg_solve_handle_provide (struct pkg_solve_problem *problem,
 			}
 		}
 		else {
-			providefound = stringlist_contains(&pkg->provides, pr->provide);
+			providefound = charv_contains(&pkg->provides, pr->provide, false);
 		}
 
 		if (!providefound && !libfound) {
@@ -499,6 +508,46 @@ pkg_solve_add_require_rule(struct pkg_solve_problem *problem,
 	return (EPKG_OK);
 }
 
+static int
+pkg_solve_add_vital_rule(struct pkg_solve_problem *problem,
+		struct pkg_solve_variable *var)
+{
+	struct pkg_solve_variable *cur_var, *local_var = NULL, *remote_var = NULL;
+	struct pkg_solve_rule *rule = NULL;
+	struct pkg *pkg;
+
+	LL_FOREACH(var, cur_var) {
+		pkg = cur_var->unit->pkg;
+
+		if (pkg->type == PKG_INSTALLED) {
+			local_var = cur_var;
+		} else {
+			remote_var = cur_var;
+		}
+	}
+
+	if (local_var && remote_var) {
+		/* Vital upgrade rule: ( L | R ) must be true */
+		dbg(4, "Add vital rule: want either %s(l) or %s(r)", local_var->unit->pkg->uid, remote_var->unit->pkg->uid);
+		rule = pkg_solve_rule_new(PKG_RULE_VITAL);
+		/* L */
+		pkg_solve_item_new(rule, local_var, 1);
+		/* R */
+		pkg_solve_item_new(rule, remote_var, 1);
+	} else if(local_var) {
+		/* Vital keep local rule: ( L ) must be true */
+		dbg(4, "Add vital rule: want %s(l) to stay", local_var->unit->pkg->uid);
+		rule = pkg_solve_rule_new(PKG_RULE_VITAL);
+		/* L */
+		pkg_solve_item_new(rule, local_var, 1);
+	}
+
+	if (rule)
+		tll_push_front(problem->rules, rule);
+
+	return (EPKG_OK);
+}
+
 static struct pkg_solve_variable *
 pkg_solve_find_var_in_chain(struct pkg_solve_variable *var,
 	struct pkg_job_universe_item *item)
@@ -637,6 +686,7 @@ pkg_solve_process_universe_variable(struct pkg_solve_problem *problem,
 	struct pkg_job_request *jreq = NULL;
 	bool chain_added = false;
 	bool force = j->flags & PKG_FLAG_FORCE;
+	bool force_overrides_vital = pkg_object_bool(pkg_config_get("FORCE_CAN_REMOVE_VITAL"));
 
 	LL_FOREACH(var, cur_var) {
 		pkg = cur_var->unit->pkg;
@@ -671,22 +721,31 @@ pkg_solve_process_universe_variable(struct pkg_solve_problem *problem,
 		}
 
 		/* Shlibs */
-		tll_foreach(pkg->shlibs_required, s) {
+		vec_foreach(pkg->shlibs_required, i) {
+			const char *s = pkg->shlibs_required.d[i];
 			/* Ignore 32 bit libraries */
-			if (j->ignore_compat32 && str_ends_with(s->item, ":32"))
+			if (j->ignore_compat32 && str_ends_with(s, ":32"))
 				continue;
-			if (pkghash_get(j->system_shlibs, s->item) != NULL) {
+			if (pkghash_get(j->system_shlibs, s) != NULL) {
 				/* The shlib is provided by the system */
 				continue;
 			}
 			if (pkg_solve_add_require_rule(problem, cur_var,
-			    s->item, cur_var->assumed_reponame) != EPKG_OK) {
+			    s, cur_var->assumed_reponame) != EPKG_OK) {
 				continue;
 			}
 		}
-		tll_foreach(pkg->requires, r) {
+		vec_foreach(pkg->requires, i) {
 			if (pkg_solve_add_require_rule(problem, cur_var,
-			    r->item, cur_var->assumed_reponame) != EPKG_OK) {
+			    pkg->requires.d[i], cur_var->assumed_reponame) != EPKG_OK) {
+				continue;
+			}
+		}
+
+		/* Vital flag */
+		bool add_vital = force_overrides_vital ? !force : true;
+		if (pkg->vital && add_vital) {
+			if (pkg_solve_add_vital_rule(problem, cur_var) != EPKG_OK) {
 				continue;
 			}
 		}
@@ -1132,7 +1191,7 @@ reiterate:
 		for (i = 0; i < problem->nvars; i ++) {
 			struct pkg_solve_variable *var = &problem->variables[i];
 
-			if (var->flags & PKG_VAR_TOP || var->unit->pkg->vital) {
+			if (var->flags & PKG_VAR_TOP) {
 				if (var->flags & PKG_VAR_FAILED) {
 					var->flags ^= PKG_VAR_INSTALL | PKG_VAR_FAILED;
 				}
