@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2011-2024 Baptiste Daroussin <bapt@FreeBSD.org>
+ * Copyright (c) 2011-2025 Baptiste Daroussin <bapt@FreeBSD.org>
  * Copyright (c) 2011-2012 Julien Laffaye <jlaffaye@FreeBSD.org>
  * Copyright (c) 2011 Will Andrews <will@FreeBSD.org>
  * Copyright (c) 2011 Philippe Pepiot <phil@philpep.org>
@@ -50,7 +50,6 @@
 #include "private/pkgdb.h"
 #include "private/utils.h"
 #include "private/pkg_deps.h"
-#include "tllist.h"
 #include "pkg/vec.h"
 
 #include "private/db_upgrades.h"
@@ -159,13 +158,25 @@ pkgdb_regex_delete(void *p)
 void
 pkgdb_now(sqlite3_context *ctx, int argc, __unused sqlite3_value **argv)
 {
+	int64_t t = (int64_t)time(NULL);
+	const char *date_env;
+
 	if (argc != 0) {
 		sqlite3_result_error(ctx, "Invalid usage of now() "
 		    "no arguments expected\n", -1);
 		return;
 	}
 
-	sqlite3_result_int64(ctx, (int64_t)time(NULL));
+	if ((date_env = getenv("PKG_INSTALL_EPOCH")) != NULL)
+	{
+		const char *errstr = NULL;
+		int64_t temp_t = strtonum(date_env, 0, INT64_MAX, &errstr);
+		if (errstr == NULL) {
+			t = temp_t;
+		}
+	}
+
+	sqlite3_result_int64(ctx, t);
 }
 
 static void
@@ -835,7 +846,7 @@ pkgdb_open_repos(struct pkgdb *db, const char *reponame)
 			/* We need read only access here */
 			if (r->ops->open(r, R_OK) == EPKG_OK) {
 				r->ops->init(r);
-				tll_push_front(db->repos, r);
+				vec_push(&db->repos, r);
 			} else
 				pkg_emit_error("Repository %s cannot be opened."
 				    " 'pkg update' required", r->name);
@@ -1135,7 +1146,7 @@ pkgdb_close(struct pkgdb *db)
 
 	if (db->sqlite != NULL) {
 
-		tll_free_and_free(db->repos, pkgdb_free_repo);
+		vec_free_and_free(&db->repos, pkgdb_free_repo);
 
 		if (!sqlite3_db_readonly(db->sqlite, "main"))
 			pkg_plugins_hook_run(PKG_PLUGIN_HOOK_PKGDB_CLOSE_RW, NULL, db);
@@ -1821,10 +1832,10 @@ pkgdb_register_pkg(struct pkgdb *db, struct pkg *pkg, int forced,
 	 * Insert categories
 	 */
 
-	tll_foreach(pkg->categories, c) {
-		ret = run_prstmt(CATEGORY1, c->item);
+	vec_foreach(pkg->categories, i) {
+		ret = run_prstmt(CATEGORY1, pkg->categories.d[i]);
 		if (ret == SQLITE_DONE)
-			ret = run_prstmt(CATEGORY2, package_id, c->item);
+			ret = run_prstmt(CATEGORY2, package_id, pkg->categories.d[i]);
 		if (ret != SQLITE_DONE) {
 			ERROR_STMT_SQLITE(s, STMT(CATEGORY2));
 			goto cleanup;
@@ -1835,11 +1846,11 @@ pkgdb_register_pkg(struct pkgdb *db, struct pkg *pkg, int forced,
 	 * Insert licenses
 	 */
 
-	tll_foreach(pkg->licenses, l) {
-		if (run_prstmt(LICENSES1, l->item)
+	vec_foreach(pkg->licenses, i) {
+		if (run_prstmt(LICENSES1, pkg->licenses.d[i])
 		    != SQLITE_DONE
 		    ||
-		    run_prstmt(LICENSES2, package_id, l->item)
+		    run_prstmt(LICENSES2, package_id, pkg->licenses.d[i])
 		    != SQLITE_DONE) {
 			ERROR_STMT_SQLITE(s, STMT(LICENSES2));
 			goto cleanup;
@@ -1850,11 +1861,11 @@ pkgdb_register_pkg(struct pkgdb *db, struct pkg *pkg, int forced,
 	 * Insert users
 	 */
 
-	tll_foreach(pkg->users, u) {
-		if (run_prstmt(USERS1, u->item)
+	vec_foreach(pkg->users, i) {
+		if (run_prstmt(USERS1, pkg->users.d[i])
 		    != SQLITE_DONE
 		    ||
-		    run_prstmt(USERS2, package_id, u->item)
+		    run_prstmt(USERS2, package_id, pkg->users.d[i])
 		    != SQLITE_DONE) {
 			ERROR_STMT_SQLITE(s, STMT(USERS2));
 			goto cleanup;
@@ -1865,11 +1876,11 @@ pkgdb_register_pkg(struct pkgdb *db, struct pkg *pkg, int forced,
 	 * Insert groups
 	 */
 
-	tll_foreach(pkg->groups, g) {
-		if (run_prstmt(GROUPS1, g->item)
+	vec_foreach(pkg->groups, i) {
+		if (run_prstmt(GROUPS1, pkg->groups.d[i])
 		    != SQLITE_DONE
 		    ||
-		    run_prstmt(GROUPS2, package_id, g->item)
+		    run_prstmt(GROUPS2, package_id, pkg->groups.d[i])
 		    != SQLITE_DONE) {
 			ERROR_STMT_SQLITE(s, STMT(GROUPS2));
 			goto cleanup;
@@ -1974,10 +1985,10 @@ pkgdb_insert_lua_scripts(struct pkg *pkg, int64_t package_id, sqlite3 *s)
 	int64_t			 i;
 
 	for (i = 0; i < PKG_NUM_LUA_SCRIPTS; i++) {
-		tll_foreach(pkg->lua_scripts[i], script) {
-			if (run_prstmt(LUASCRIPT1, script->item) != SQLITE_DONE
+		vec_foreach(pkg->lua_scripts[i], j) {
+			if (run_prstmt(LUASCRIPT1, pkg->lua_scripts[i].d[j]) != SQLITE_DONE
 			    ||
-			    run_prstmt(LUASCRIPT2, script->item, package_id, i) != SQLITE_DONE) {
+			    run_prstmt(LUASCRIPT2, pkg->lua_scripts[i].d[j], package_id, i) != SQLITE_DONE) {
 				ERROR_STMT_SQLITE(s, STMT(LUASCRIPT2));
 				return (EPKG_FATAL);
 			}
@@ -1989,11 +2000,11 @@ pkgdb_insert_lua_scripts(struct pkg *pkg, int64_t package_id, sqlite3 *s)
 int
 pkgdb_update_shlibs_required(struct pkg *pkg, int64_t package_id, sqlite3 *s)
 {
-	tll_foreach(pkg->shlibs_required, r) {
-		if (run_prstmt(SHLIBS1, r->item)
+	vec_foreach(pkg->shlibs_required, i) {
+		if (run_prstmt(SHLIBS1, pkg->shlibs_required.d[i])
 		    != SQLITE_DONE
 		    ||
-		    run_prstmt(SHLIBS_REQD, package_id, r->item)
+		    run_prstmt(SHLIBS_REQD, package_id, pkg->shlibs_required.d[i])
 		    != SQLITE_DONE) {
 			ERROR_STMT_SQLITE(s, STMT(SHLIBS_REQD));
 			return (EPKG_FATAL);
@@ -2022,11 +2033,11 @@ pkgdb_update_config_file_content(struct pkg *p, sqlite3 *s)
 int
 pkgdb_update_shlibs_provided(struct pkg *pkg, int64_t package_id, sqlite3 *s)
 {
-	tll_foreach(pkg->shlibs_provided, r) {
-		if (run_prstmt(SHLIBS1, r->item)
+	vec_foreach(pkg->shlibs_provided, i) {
+		if (run_prstmt(SHLIBS1, pkg->shlibs_provided.d[i])
 		    != SQLITE_DONE
 		    ||
-		    run_prstmt(SHLIBS_PROV, package_id, r->item)
+		    run_prstmt(SHLIBS_PROV, package_id, pkg->shlibs_provided.d[i])
 		    != SQLITE_DONE) {
 			ERROR_STMT_SQLITE(s, STMT(SHLIBS_PROV));
 			return (EPKG_FATAL);
@@ -2039,11 +2050,11 @@ pkgdb_update_shlibs_provided(struct pkg *pkg, int64_t package_id, sqlite3 *s)
 int
 pkgdb_update_requires(struct pkg *pkg, int64_t package_id, sqlite3 *s)
 {
-	tll_foreach(pkg->requires, r) {
-		if (run_prstmt(REQUIRE, r->item)
+	vec_foreach(pkg->requires, i) {
+		if (run_prstmt(REQUIRE, pkg->requires.d[i])
 		    != SQLITE_DONE
 		    ||
-		    run_prstmt(PKG_REQUIRE, package_id, r->item)
+		    run_prstmt(PKG_REQUIRE, package_id, pkg->requires.d[i])
 		    != SQLITE_DONE) {
 			ERROR_STMT_SQLITE(s, STMT(PKG_REQUIRE));
 			return (EPKG_FATAL);
@@ -2056,11 +2067,11 @@ pkgdb_update_requires(struct pkg *pkg, int64_t package_id, sqlite3 *s)
 int
 pkgdb_update_provides(struct pkg *pkg, int64_t package_id, sqlite3 *s)
 {
-	tll_foreach(pkg->provides, p) {
-		if (run_prstmt(PROVIDE, p->item)
+	vec_foreach(pkg->provides, i) {
+		if (run_prstmt(PROVIDE, pkg->provides.d[i])
 		    != SQLITE_DONE
 		    ||
-		    run_prstmt(PKG_PROVIDE, package_id, p->item)
+		    run_prstmt(PKG_PROVIDE, package_id, pkg->provides.d[i])
 		    != SQLITE_DONE) {
 			ERROR_STMT_SQLITE(s, STMT(PKG_PROVIDE));
 			return (EPKG_FATAL);
@@ -2075,8 +2086,8 @@ pkgdb_insert_annotations(struct pkg *pkg, int64_t package_id, sqlite3 *s)
 {
 	struct pkg_kv	*kv;
 
-	tll_foreach(pkg->annotations, k) {
-		kv = k->item;
+	vec_foreach(pkg->annotations, i) {
+		kv = pkg->annotations.d[i];
 		if (run_prstmt(ANNOTATE1, kv->key)
 		    != SQLITE_DONE
 		    ||
@@ -2873,14 +2884,14 @@ pkgdb_stats(struct pkgdb *db, pkg_stats_t type)
 	case PKG_STATS_REMOTE_UNIQUE:
 	case PKG_STATS_REMOTE_COUNT:
 	case PKG_STATS_REMOTE_SIZE:
-		tll_foreach(db->repos, rit) {
-			if (rit->item->ops->stat != NULL)
-				stats += rit->item->ops->stat(rit->item, type);
+		vec_foreach(db->repos, i) {
+			if (db->repos.d[i]->ops->stat != NULL)
+				stats += db->repos.d[i]->ops->stat(db->repos.d[i], type);
 		}
 		return (stats);
 		break;
 	case PKG_STATS_REMOTE_REPOS:
-		return (tll_length(db->repos));
+		return (vec_len(&db->repos));
 		break;
 	}
 
@@ -2912,30 +2923,31 @@ pkgdb_begin_solver(struct pkgdb *db)
 		"CREATE INDEX pkg_digest_id ON packages(name, manifestdigest);";
 	struct pkgdb_it *it;
 	struct pkg *p = NULL;
-	tll(struct pkg *) pkglist = tll_init();
+	pkgs_t pkglist;
 	int rc = EPKG_OK;
 	int64_t cnt = 0, cur = 0;
 
+	vec_init(&pkglist);
 	it = pkgdb_query_cond(db, " WHERE manifestdigest IS NULL OR manifestdigest==''",
 		NULL, MATCH_ALL);
 	if (it != NULL) {
 		while (pkgdb_it_next(it, &p, PKG_LOAD_BASIC|PKG_LOAD_OPTIONS) == EPKG_OK) {
 			pkg_checksum_calculate(p, NULL, false, true, false);
-			tll_push_front(pkglist, p);
+			vec_push(&pkglist, p);
 			p = NULL;
 			cnt ++;
 		}
 		pkgdb_it_free(it);
 
-		if (tll_length(pkglist) > 0) {
+		if (vec_len(&pkglist) > 0) {
 			rc = sql_exec(db->sqlite, update_digests_sql);
 			if (rc != EPKG_OK) {
 				ERROR_SQLITE(db->sqlite, update_digests_sql);
 			}
 			else {
 				pkg_emit_progress_start("Updating database digests format");
-				tll_foreach(pkglist, pit) {
-					p = pit->item;
+				vec_foreach(pkglist, i) {
+					p = pkglist.d[i];
 					pkg_emit_progress_tick(cur++, cnt);
 					rc = run_prstmt(UPDATE_DIGEST, p->digest, p->id);
 					if (rc != SQLITE_DONE) {
@@ -2956,7 +2968,7 @@ pkgdb_begin_solver(struct pkgdb *db)
 		if (rc == EPKG_OK)
 			rc = sql_exec(db->sqlite, solver_sql);
 
-		tll_free_and_free(pkglist, pkg_free);
+		vec_free_and_free(&pkglist, pkg_free);
 	} else {
 		rc = sql_exec(db->sqlite, solver_sql);
 	}
@@ -3018,3 +3030,53 @@ pkgdb_debug(int level, sqlite3_stmt *stmt)
 	dbg(level, "running: '%s'", str);
 	sqlite3_free(str);
 }
+
+bool
+pkgdb_is_shlib_provided(struct pkgdb *db, const char *req)
+{
+	sqlite3_stmt *stmt;
+	int ret;
+	bool found = false;
+
+	const char *sql = ""
+		"select package_id from pkg_shlibs_provided INNER JOIN shlibs "
+		"on pkg_shlibs_provided.shlib_id = shlibs.id "
+		"where shlibs.name=?1" ;
+
+	stmt = prepare_sql(db->sqlite, sql);
+	if (stmt == NULL)
+		return (false);
+
+	sqlite3_bind_text(stmt, 1, req, -1, SQLITE_TRANSIENT);
+	ret = sqlite3_step(stmt);
+	if (ret == SQLITE_ROW)
+		found = true;
+
+	sqlite3_finalize(stmt);
+	return (found);
+}
+
+bool
+pkgdb_is_provided(struct pkgdb *db, const char *req)
+{
+	sqlite3_stmt *stmt;
+	int ret;
+	bool found = false;
+
+	const char *sql = ""
+		"select package_id from pkg_provides INNER JOIN provides "
+		"on pkg_provides.provide_id = provides.id "
+		"where provides.provide = ?1" ;
+
+	stmt = prepare_sql(db->sqlite, sql);
+	if (stmt == NULL)
+		return (false);
+
+	sqlite3_bind_text(stmt, 1, req, -1, SQLITE_TRANSIENT);
+	ret = sqlite3_step(stmt);
+	if (ret == SQLITE_ROW)
+		found = true;
+
+	sqlite3_finalize(stmt);
+	return (found);
+}
