@@ -1,5 +1,5 @@
 /*-
- * Copyright (c) 2011-2023 Baptiste Daroussin <bapt@FreeBSD.org>
+ * Copyright (c) 2011-2025 Baptiste Daroussin <bapt@FreeBSD.org>
  * Copyright (c) 2011-2012 Julien Laffaye <jlaffaye@FreeBSD.org>
  * Copyright (c) 2011 Will Andrews <will@FreeBSD.org>
  * Copyright (c) 2011 Philippe Pepiot <phil@philpep.org>
@@ -10,28 +10,8 @@
  * Copyright (c) 2013-2017 Vsevolod Stakhov <vsevolod@FreeBSD.org>
  * Copyright (c) 2023 Serenity Cyber Security, LLC
  *                    Author: Gleb Popov <arrowd@FreeBSD.org>
- * All rights reserved.
  *
- * Redistribution and use in source and binary forms, with or without
- * modification, are permitted provided that the following conditions
- * are met:
- * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer
- *    in this position and unchanged.
- * 2. Redistributions in binary form must reproduce the above copyright
- *    notice, this list of conditions and the following disclaimer in the
- *    documentation and/or other materials provided with the distribution.
- *
- * THIS SOFTWARE IS PROVIDED BY THE AUTHOR(S) ``AS IS'' AND ANY EXPRESS OR
- * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
- * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
- * IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT, INDIRECT,
- * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
- * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
- * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
- * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ * SPDX-License-Identifier: BSD-2-Clause
  */
 
 #ifdef HAVE_CONFIG_H
@@ -588,7 +568,7 @@ pkgdb_load_group(sqlite3 *sqlite, struct pkg *pkg)
 static int
 addshlib_required_raw(struct pkg *pkg, const char *name)
 {
-	tll_push_back(pkg->shlibs_required, xstrdup(name));
+	vec_push(&pkg->shlibs_required, xstrdup(name));
 	return (EPKG_OK);
 }
 
@@ -600,7 +580,7 @@ pkgdb_load_shlib_required(sqlite3 *sqlite, struct pkg *pkg)
 		"  FROM pkg_shlibs_required, shlibs AS s"
 		"  WHERE package_id = ?1"
 		"    AND shlib_id = s.id"
-		"  ORDER by name DESC";
+		"  ORDER by name ASC";
 
 	assert(pkg != NULL);
 
@@ -611,7 +591,7 @@ pkgdb_load_shlib_required(sqlite3 *sqlite, struct pkg *pkg)
 static int
 addshlib_provided_raw(struct pkg *pkg, const char *name)
 {
-	tll_push_back(pkg->shlibs_provided, xstrdup(name));
+	vec_push(&pkg->shlibs_provided, xstrdup(name));
 	return (EPKG_OK);
 }
 
@@ -623,7 +603,7 @@ pkgdb_load_shlib_provided(sqlite3 *sqlite, struct pkg *pkg)
 		"  FROM pkg_shlibs_provided, shlibs AS s"
 		"  WHERE package_id = ?1"
 		"    AND shlib_id = s.id"
-		"  ORDER by name DESC";
+		"  ORDER by name ASC";
 
 	assert(pkg != NULL);
 
@@ -902,13 +882,15 @@ populate_pkg(sqlite3_stmt *stmt, struct pkg *pkg) {
 						struct pkg_message *message;
 						message = xcalloc(1, sizeof(*message));
 						message->str = xstrdup(msg);
-						tll_push_back(pkg->message, message);
+						vec_push(&pkg->message, message);
 					}
 				}
 				break;
 			case PKG_ATTR_NAME:
 				free(pkg->name);
 				pkg->name = xstrdup(sqlite3_column_text(stmt, icol));
+				free(pkg->uid);
+				pkg->uid = xstrdup(sqlite3_column_text(stmt, icol));
 				break;
 			case PKG_ATTR_OLD_VERSION:
 				free(pkg->old_version);
@@ -1136,16 +1118,15 @@ pkgdb_it_next(struct pkgdb_it *it, struct pkg **pkg_p, unsigned flags)
 			return (r);
 	}
 
-	if (tll_length(it->remote) != 0) {
-		if (it->opq_it == NULL)
-			it->opq_it = it->remote.head;
-		__typeof__(*(it->remote).head) *lit = it->opq_it;
-		rit = lit->item;
+	if (vec_len(&it->remote) != 0) {
+		if (it->remote_pos >= it->remote.len)
+			it->remote_pos = 0;
+		struct pkg_repo_it *rit = it->remote.d[it->remote_pos];
 		ret = rit->ops->next(rit, pkg_p, flags);
 		if (ret != EPKG_OK) {
-			if (it->opq_it == it->remote.tail)
+			if (it->remote_pos == it->remote.len -1 )
 				return (EPKG_END);
-			it->opq_it = lit->next;
+			it->remote_pos++;
 			return (pkgdb_it_next(it, pkg_p, flags));
 		}
 
@@ -1200,8 +1181,8 @@ pkgdb_it_reset(struct pkgdb_it *it)
 	if (it->local != NULL) {
 		pkgdb_sqlite_it_reset(it->local);
 	}
-	tll_foreach(it->remote, cur) {
-		cur->item->ops->reset(cur->item);
+	vec_foreach(it->remote, i) {
+		it->remote.d[i]->ops->reset(it->remote.d[i]);
 	}
 }
 
@@ -1215,7 +1196,7 @@ pkgdb_it_free(struct pkgdb_it *it)
 		pkgdb_sqlite_it_free(it->local);
 		free(it->local);
 	}
-	tll_free_and_free(it->remote, remote_free);
+	vec_free_and_free(&it->remote, remote_free);
 
 	free(it);
 }
@@ -1239,7 +1220,7 @@ pkgdb_it_new_sqlite(struct pkgdb *db, sqlite3_stmt *s, int type, short flags)
 
 	it->local->flags = flags;
 	it->local->finished = 0;
-	it->opq_it = it->remote.head;
+	it->remote_pos = 0;
 
 	return (it);
 }
@@ -1259,7 +1240,7 @@ pkgdb_it_new_repo(struct pkgdb *db)
 void
 pkgdb_it_repo_attach(struct pkgdb_it *it, struct pkg_repo_it *rit)
 {
-	tll_push_front(it->remote, rit);
+	vec_push(&it->remote, rit);
 }
 
 int
@@ -1285,10 +1266,10 @@ pkgdb_ensure_loaded(struct pkgdb *db, struct pkg *pkg, unsigned flags)
 	if (pkg->type == PKG_INSTALLED)
 		return (pkgdb_ensure_loaded_sqlite(db->sqlite, pkg, flags));
 
-	tll_foreach(db->repos, cur) {
-		if (cur->item == pkg->repo) {
-			if (cur->item->ops->ensure_loaded) {
-				return (cur->item->ops->ensure_loaded(cur->item,
+	vec_foreach(db->repos, i) {
+		if (db->repos.d[i] == pkg->repo) {
+			if (db->repos.d[i]->ops->ensure_loaded) {
+				return (db->repos.d[i]->ops->ensure_loaded(db->repos.d[i],
 				    pkg, flags));
 			}
 		}
