@@ -99,7 +99,7 @@
 } while (0)
 #define DL_FREE(head, free_func) DL_FREE2(head, free_func, prev, next)
 
-typedef tll(struct pkg_kv *) kvlist_t;
+typedef vec_t(struct pkg_kv *) kvlist_t;
 
 typedef enum {
 	IPALL = 0,
@@ -112,17 +112,17 @@ struct pkg_kvlist {
 };
 
 struct pkg_stringlist {
-	stringlist_t *list;
+	charv_t *list;
 };
 
 struct pkg_kvlist_iterator {
 	kvlist_t *list;
-	void *cur;
+	size_t pos;
 };
 
 struct pkg_stringlist_iterator {
-	stringlist_t *list;
-	void *cur;
+	charv_t *list;
+	size_t pos;
 };
 
 struct pkg_ctx {
@@ -153,6 +153,8 @@ struct pkg_ctx {
 	ip_version_t ip;
 	struct pkg_abi abi;
 	bool ischrooted;
+	bool no_version_for_deps;
+	bool track_linux_compat_shlibs;
 };
 
 extern struct pkg_ctx ctx;
@@ -171,16 +173,17 @@ struct pkg_repo;
 struct url;
 struct fetcher;
 struct pkg_message;
-typedef tll(struct pkg_message *) messages_t;
+typedef vec_t(struct pkg_message *) messages_t;
 
 struct pkg {
 	bool		 direct;
 	bool		 locked;
 	bool		 automatic;
 	bool		 vital;
+	bool		 list_sorted;
 	int64_t		 id;
 	xstring		*scripts[PKG_NUM_SCRIPTS];
-	stringlist_t	 lua_scripts[PKG_NUM_LUA_SCRIPTS];
+	charv_t	 lua_scripts[PKG_NUM_LUA_SCRIPTS];
 	char			*name;
 	char			*origin;
 	char			*version;
@@ -212,33 +215,34 @@ struct pkg {
 	struct pkg_dep		*depends;
 	pkghash			*rdepshash;
 	struct pkg_dep		*rdepends;
-	stringlist_t		 categories;
-	stringlist_t		 licenses;
+	charv_t		 categories;
+	charv_t		 licenses;
 	pkghash			*filehash;
 	struct pkg_file		*files;
 	pkghash			*dirhash;
 	struct pkg_dir		*dirs;
 	pkghash			*optionshash;
 	struct pkg_option	*options;
-	stringlist_t		 users;
-	stringlist_t		 groups;
-	stringlist_t		 shlibs_required;
-	stringlist_t		 shlibs_provided;
+	charv_t		 users;
+	charv_t		 groups;
+	charv_t		 shlibs_required;
+	charv_t		 shlibs_provided;
 	pkghash			*conflictshash;
 	struct pkg_conflict	*conflicts;
-	stringlist_t		 provides;
-	stringlist_t		 requires;
+	charv_t		 provides;
+	charv_t		 requires;
 	pkghash			*config_files_hash;
 	struct pkg_config_file	*config_files;
 	kvlist_t		 annotations;
 	unsigned			flags;
 	int		rootfd;
 	char		rootpath[MAXPATHLEN];
-	stringlist_t	dir_to_del;
+	charv_t	dir_to_del;
 	pkg_t		 type;
 	struct pkg_repo		*repo;
 };
 typedef tll(struct pkg *) pkg_chain_t;
+typedef vec_t(struct pkg *) pkgs_t;
 
 typedef enum {
 	SCRIPT_UNKNOWN = 0,
@@ -555,7 +559,7 @@ struct pkg_repo {
 	struct pkg_repo *next, *prev;
 };
 
-typedef tll(struct action *) actions_t;
+typedef vec_t(struct action *) actions_t;
 struct keyword {
 	char *keyword;
 	actions_t actions;
@@ -624,6 +628,7 @@ int pkg_delete(struct pkg *pkg, struct pkg *rpkg, struct pkgdb *db, int flags,
     struct triggers *);
 #define PKG_DELETE_UPGRADE	(1 << 1)	/* delete as a split upgrade */
 #define PKG_DELETE_NOSCRIPT	(1 << 2)	/* don't run delete scripts */
+#define PKG_DELETE_NOEXEC	(1 << 3)	/* don't run delete scripts which execute things*/
 
 int pkg_fetch_file_to_fd(struct pkg_repo *repo, int dest, struct fetch_item *,
     bool silent);
@@ -658,9 +663,9 @@ int pkg_repo_load_fingerprints(struct pkg_repo *repo);
 
 int pkg_start_stop_rc_scripts(struct pkg *, pkg_rc_attr attr);
 
-int pkg_script_run(struct pkg *, pkg_script type, bool upgrade);
+int pkg_script_run(struct pkg *, pkg_script type, bool upgrade, bool noexec);
 int pkg_lua_script_run(struct pkg *, pkg_lua_script type, bool upgrade);
-ucl_object_t *pkg_lua_script_to_ucl(stringlist_t *);
+ucl_object_t *pkg_lua_script_to_ucl(charv_t *);
 int pkg_script_run_child(int pid, int *pstat, int inputfd, const char* script_name);
 
 int pkg_open2(struct pkg **p, struct archive **a, struct archive_entry **ae,
@@ -787,7 +792,7 @@ int pkg_adddir(struct pkg *pkg, const char *path, bool check_duplicates);
 int pkg_adddir_attr(struct pkg *pkg, const char *path, const char *uname,
     const char *gname, mode_t perm, u_long fflags, bool check_duplicates);
 
-int pkg_addstring(stringlist_t *s, const char *value, const char *title);
+int pkg_addstring(charv_t *s, const char *value, const char *title);
 int pkg_kv_add(kvlist_t *kv, const char *key, const char *value, const char *title);
 const char *pkg_kv_get(const kvlist_t *kv, const char *key);
 int pkg_adduser(struct pkg *pkg, const char *name);
@@ -854,7 +859,6 @@ void trigger_is_it_a_cleanup(struct triggers *t, const char *path);
 void trigger_free(struct trigger *);
 void append_touched_dir(const char *path);
 void append_touched_file(const char *path);
-bool stringlist_contains(stringlist_t *l, const char *name);
 
 int pkg_parse_manifest_ucl(struct pkg *pkg, ucl_object_t *o);
 int pkg_get_reposdirfd(void);
@@ -862,5 +866,6 @@ char * expand_plist_variables(const char *in, kvlist_t *vars);
 
 int scan_system_shlibs(pkghash **system_shlibs, const char *rootdir);
 void pkg_lists_sort(struct pkg *p);
+void pkg_cleanup_shlibs_required(struct pkg *pkg, charv_t *internal_provided);
 
 #endif
